<?php

namespace Drupal\domain_traversal;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\domain\Entity\Domain;
use Drupal\domain_access\DomainAccessManagerInterface;
use Drupal\user\UserInterface;

class DomainTraversal implements DomainTraversalInterface {

  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\domain_access\DomainAccessManagerInterface|null $domainAccessManager
   */
  public function __construct(
    private EntityTypeManagerInterface $entityTypeManager,
    private ?DomainAccessManagerInterface $domainAccessManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getAccountTraversableDomainIds(UserInterface $account): array {
    $domainStorage = $this->entityTypeManager->getStorage('domain');
    $accountDomainIds = NULL;
    if ($this->domainAccessManager && !$this->accountMayTraverseAllDomains($account)) {
      $accountDomainIds = array_keys($this->domainAccessManager->getAccessValues($account));
    }

    $domainIds = [];

    /** @var \Drupal\domain\DomainInterface $domain */
    foreach ($domainStorage->loadMultiple($accountDomainIds) as $domain) {
      if (!$domain->status()) {
        continue;
      }
      $domainIds[$domain->id()] = $domain->id();
    }

    return $domainIds;
  }

  /**
   * {@inheritdoc}
   */
  public function accountMayTraverseDomain(UserInterface $account, Domain $domain): bool {
    if ($account->isAnonymous()) {
      return FALSE;
    }

    if ($this->accountMayTraverseAllDomains($account)) {
      return TRUE;
    }

    $domainIds = $this->getAccountTraversableDomainIds($account);

    return isset($domainIds[$domain->id()]);
  }

  /**
   * {@inheritdoc}
   */
  public function accountMayTraverseAllDomains(UserInterface $account): bool {
    if ($account->isAnonymous()) {
      return FALSE;
    }

    if ($account->hasPermission('traverse all domains')) {
      return TRUE;
    }

    if ($this->domainAccessManager && !empty($this->domainAccessManager->getAllValue($account))) {
      return TRUE;
    }

    return FALSE;
  }

}
