<?php

namespace Drupal\domain_traversal\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

class DomainTraversalMenuLink extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(protected EntityTypeManagerInterface $entityTypeManager) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $domainStorage = $this->entityTypeManager->getStorage('domain');

    $this->derivatives = [];

    /** @var \Drupal\domain\Entity\Domain[] $domains */
    $domains = $domainStorage->loadMultiple();
    foreach ($domains as $domain) {
      if (!$domain->status()) {
        continue;
      }

      $domainId = $domain->id();

      $args = [
        '@domain_name' => Markup::create($domain->get('name')),
        '@domain_hostname' => $domain->getPath(),
      ];

      $this->derivatives['domain_traversal.traverse.' . $domainId] = [
          'route_name' => 'domain_traversal.traverse',
          'route_parameters' => [
            'domain' => $domainId,
          ],
          'title' => Markup::create($domain->get('name')),
          'description' => $this->t('Traverse to this domain: @domain_hostname', $args),
          'parent' => 'domain_traversal',
          'menu_name' => 'admin',
          'weight' => $domain->getWeight(),
        ] + $base_plugin_definition;
    }

    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
